@@ -1,6 +1,6 @@
 /*--------------------------------------------------------------------
  *
- *  Copyright (c) 2016-2017 by Dongdong Tian
+ *  Copyright (c) 2016-2018 by Dongdong Tian
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  * Brief synopsis: pssac will plot seismograms in SAC format on maps
  */
 
-/* 
+/*
 The SAC I/O functions are initially written by Prof. Lupei Zhu,
 and modified by Dongdong Tian.
 
@@ -30,15 +30,15 @@ provided that the above copyright notice appear in all copies, that both
 that copyright notice and this permission notice appear in supporting documentation.
 */
 
+#include "gmt_dev.h"
+#include "sacio.h"
+
 #define THIS_MODULE_NAME	"pssac"
 #define THIS_MODULE_LIB		"meca"
 #define THIS_MODULE_PURPOSE	"Plot seismograms in SAC format on maps"
 #define THIS_MODULE_KEYS	">XO,RG-"
-
-#include "gmt_dev.h"
-#include "sacio.h"
-
-#define GMT_PROG_OPTIONS "->BJKOPRUVXYcht"
+#define THIS_MODULE_NEEDS	"RJ"
+#define THIS_MODULE_OPTIONS "->BJKOPRUVXYht" GMT_OPT("c")
 
 /* Control structure for pssac */
 
@@ -83,9 +83,10 @@ struct PSSAC_CTRL {
 	struct PSSAC_Q {	/* -Q */
 		bool active;
 	} Q;
-	struct PSSAC_S {	/* -S<sec_per_measure> */
+	struct PSSAC_S {	/* -S[i]<scale>[<unit>] */
 		bool active;
-		double sec_per_measure;
+		double factor;
+		char unit;
 	} S;
 	struct PSSAC_T {   /* -T+t<n>+r<reduce_vel>+s<shift> */
 		bool active;
@@ -96,6 +97,7 @@ struct PSSAC_CTRL {
 		double shift;
 	} T;
 	struct PSSAC_W {	/* -W<pen> */
+		bool active;
 		struct GMT_PEN pen;
 	} W;
 };
@@ -135,9 +137,9 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "usage: pssac [<saclist>|<SACfiles>] %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [-C[<t0>/<t1>]] [-D<dx>[/<dy>]] [-Ea|b|k|d|n[<n>]|u[<n>]] [-F[i][q][r]]\n", GMT_B_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\t[-G[p|n][+g<fill>][+t<t0>/<t1>][+z<zero>]] [-K] [-M<size>[<unit>]/<alpha>] [-O] [-P] [-Q]\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t[-S<sec_per_measure>[<unit>]] [-T[+t<tmark>][+r<reduce_vel>][+s<shift>]] \n");
+	GMT_Message (API, GMT_TIME_NONE, "\t[-S[i]<scale>[<unit>]] [-T[+t<tmark>][+r<reduce_vel>][+s<shift>]] \n");
 	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [-W<pen>]\n", GMT_U_OPT, GMT_V_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s] [%s] \n\t[%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_h_OPT, GMT_t_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[%s] [%s]\n\t[%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT, GMT_h_OPT, GMT_t_OPT);
 	GMT_Message (API, GMT_TIME_NONE, "\n");
 
 	if (level == GMT_SYNOPSIS) return (EXIT_FAILURE);
@@ -170,7 +172,7 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   i: integral\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   q: square\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   r: remove mean value\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   i|q|r can repeat multiple times. -Frii will convert accerate to displacement.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   i|q|r can repeat multiple times. -Frii will convert acceleration to displacement.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   The order of i|q|r controls the order of the data processing.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-G Paint positive or negative portion of traces.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If only -G is used, default to fill the positive portion black.\n");
@@ -188,14 +190,16 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t      <alpha> > 0, multiply all traces by size*r^alpha, r is the distance range in km.\n");
 	GMT_Option (API, "O,P");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Plot traces vertically.\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t-S Specify the time scale in seconds per <unit> while plotting on geographic plots. Use PROJ_LENGTH_UNIT if <unit> is omitted.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t-S Sets time scale in seconds per <unit> while plotting on geographic plots. \n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Append c, i, or p to indicate cm, inch or points as the unit. Use PROJ_LENGTH_UNIT if <unit> is omitted.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Si<scale><unit> to give the reciprocal scale, i.e. cm per second or inch per second.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-T Time alignment. \n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   +t<tmark> align all trace along time mark. Choose <tmark> from -5(b), -4(e), -3(o), -2(a), 0-9(t0-t9).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   +r<reduce_vel> reduce velocity in km/s.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   +s<shift> shift all traces by <shift> seconds.\n");
 	GMT_Option (API, "U,V");
 	gmt_pen_syntax (API->GMT, 'W', "Set pen attributes [Default pen is %s]:", 0);
-	GMT_Option (API, "X,c,h,t");
+	GMT_Option (API, "X,h,t");
 	GMT_Option (API, ".");
 
 	return (EXIT_FAILURE);
@@ -211,7 +215,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSSAC_CTRL *Ctrl, struct GMT_O
 
 	unsigned int n_errors = 0, pos = 0;
 	int j, k;
-	size_t n_alloc = 0;
+	size_t n_alloc = 0, len;
 	char txt_a[GMT_LEN256] = {""}, txt_b[GMT_LEN256] = {""}, p[GMT_BUFSIZ] = {""};
 	char path[GMT_BUFSIZ] = {""};	/* Full path to sac file */
 	struct GMT_OPTION *opt = NULL;
@@ -269,7 +273,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSSAC_CTRL *Ctrl, struct GMT_O
 				}
 				Ctrl->G.active[k] = true;
 				pos = j;
-				while (gmt_getmodopt (GMT, opt->arg, "gtz", &pos, p)) {
+				while (gmt_getmodopt (GMT, 'G', opt->arg, "gtz", &pos, p, &n_errors) && n_errors == 0) {
 					switch (p[0]) {
 						case 'g':  /* fill */
 							if (gmt_getfill (GMT, &p[1], &Ctrl->G.fill[k])) {
@@ -287,9 +291,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSSAC_CTRL *Ctrl, struct GMT_O
 								n_errors++;
 							}
 							break;
-						default:
-							GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -G: -G+g<fill>+z<zero>+t<t0>/<t1>\n");
-							break;
+						default: break;	/* These are caught in gmt_getmodopt so break is just for Coverity */
 					}
 				}
 				break;
@@ -303,7 +305,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSSAC_CTRL *Ctrl, struct GMT_O
 				else if (j == 2) {
 					if (strcmp(txt_b, "s") == 0 ) {   /* -Msize/s */
 						// TODO
-					} 
+					}
 					else if (strcmp(txt_b, "b") == 0) {  /* -Msize/b */
 						// TODO
 					}
@@ -332,13 +334,39 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSSAC_CTRL *Ctrl, struct GMT_O
 				break;
 			case 'S':
 				Ctrl->S.active = true;
-				Ctrl->S.sec_per_measure = gmt_M_to_inch (GMT, opt->arg);
+				len = strlen (opt->arg) - 1;
+				j = (opt->arg[0] == 'i') ? 1 : 0;
+				if (strchr(GMT_DIM_UNITS, (int)opt->arg[len])) /* Recognized unit character */
+					Ctrl->S.unit = opt->arg[len];
+				else if (! (opt->arg[len] == '.' || isdigit ((int)opt->arg[len]))) {    /* Not decimal point or digit means trouble */
+					GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -S option: Unrecognized unit %c\n", opt->arg[len]);
+					n_errors++;
+				}
+				if (j == 0) {
+					Ctrl->S.factor = 1.0 / atof(&opt->arg[j]);
+				} else {
+					Ctrl->S.factor = atof(&opt->arg[j]);
+				}
+				switch (Ctrl->S.unit) { /* Adjust for possible unit selection */
+					case 'c':
+						Ctrl->S.factor *= GMT->session.u2u[GMT_CM][GMT_INCH];
+						break;
+					case 'i':
+						Ctrl->S.factor *= GMT->session.u2u[GMT_INCH][GMT_INCH];
+						break;
+					case 'p':
+						Ctrl->S.factor *= GMT->session.u2u[GMT_PT][GMT_INCH];
+						break;
+					default:
+						Ctrl->S.factor *= GMT->session.u2u[GMT->current.setting.proj_length_unit][GMT_INCH];
+						break;
+				}
 				break;
 			case 'T':
 				pos = 0;
 				Ctrl->T.active = true;
 				Ctrl->T.shift = 0.0;  /* default no shift */
-				while (gmt_getmodopt (GMT, opt->arg, "trs", &pos, p)) {
+				while (gmt_getmodopt (GMT, 'T', opt->arg, "trs", &pos, p, &n_errors) && n_errors == 0) {
 					switch (p[0]) {
 						case 't':
 							Ctrl->T.align = true;
@@ -351,14 +379,12 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSSAC_CTRL *Ctrl, struct GMT_O
 						case 's':
 							Ctrl->T.shift = atof (&p[1]);
 							break;
-						default:
-							GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -T option: -T+t<align>+r<reduce_vel>+s<shift>");
-							n_errors++;
-							break;
+						default: break;	/* These are caught in gmt_getmodopt so break is just for Coverity */
 					}
 				}
 				break;
 			case 'W':		/* Set line attributes */
+				Ctrl->W.active = true;
 				if (opt->arg[0] && gmt_getpen (GMT, &opt->arg[0], &Ctrl->W.pen)) {
 					gmt_pen_syntax (GMT, 'W', "sets pen attributes [Default pen is %s]:", 3);
 					n_errors++;
@@ -372,9 +398,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct PSSAC_CTRL *Ctrl, struct GMT_O
 
 	/* Check that the options selected are mutually consistent */
 
-	n_errors += gmt_M_check_condition (GMT, !GMT->common.R.active, "Syntax error: Must specify -R option\n");
+	n_errors += gmt_M_check_condition (GMT, !GMT->common.R.active[RSET], "Syntax error: Must specify -R option\n");
 	n_errors += gmt_M_check_condition (GMT, !GMT->common.J.active, "Syntax error: Must specify a map projection with the -J option\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->S.active && gmt_M_is_zero(Ctrl->S.sec_per_measure), "Syntax error -S option: <sec_per_measure> must be nonzero\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->S.active && gmt_M_is_zero(Ctrl->S.factor), "Syntax error -S option: <scale> must be nonzero\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.reduce && gmt_M_is_zero(Ctrl->T.reduce_vel), "Syntax error -T option: <reduce_vel> must be nonzero\n");
 	n_errors += gmt_M_check_condition (GMT, Ctrl->T.align && !(Ctrl->T.tmark >= -5 && Ctrl->T.tmark <= 9 && Ctrl->T.tmark != -1), "Syntax error -T option: <tmark> should be chosen from -5, -4, -3, -2, 0-9\n");
 
@@ -517,9 +543,9 @@ GMT_LOCAL int init_sac_list (struct GMT_CTRL *GMT, char **files, unsigned int n_
 				if (gmt_M_rec_is_eof(GMT))  /* Reached end of file */
 					break;
 			}
-			if (line == NULL) {
-				GMT_Report (GMT, GMT_MSG_NORMAL, "line == NULL where code does not account for it. Unknown consequence.\n");
-				break;
+			if (line == NULL) {	/* Crazy safety valve but it should never get here*/
+				GMT_Report (GMT->parent, GMT_MSG_NORMAL, "Internal error: input pointer is NULL where it should not be, aborting\n");
+				return (GMT_PTR_IS_NULL);
 			}
 
 			nr = sscanf (line, "%s %lf %lf %s", file, &x, &y, pen);
@@ -557,7 +583,7 @@ GMT_LOCAL int init_sac_list (struct GMT_CTRL *GMT, char **files, unsigned int n_
 }
 
 int GMT_pssac (void *V_API, int mode, void *args) {	/* High-level function that implements the pssac task */
-	bool old_is_world, free_plot_pen = false, read_from_ascii;
+	bool old_is_world, free_plot_pen = false, read_from_ascii, draw_line;
 	unsigned int n_files, *plot_pen = NULL;
 	int error = GMT_NOERROR, n, i, npts;
 	double yscale = 1.0, y0 = 0.0, x0, tref, dt, *x = NULL, *y = NULL, *xp = NULL, *yp = NULL;
@@ -582,8 +608,8 @@ int GMT_pssac (void *V_API, int mode, void *args) {	/* High-level function that
 
 	/* Parse the command-line arguments; return if errors are encountered */
 
-	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
@@ -623,8 +649,8 @@ int GMT_pssac (void *V_API, int mode, void *args) {	/* High-level function that
 	GMT_Report (API, GMT_MSG_VERBOSE, "Collecting %ld SAC files to plot.\n", n_files);
 
 	for (n = 0; n < (int)n_files; n++) {  /* Loop over all SAC files */
- 		GMT_Report (API, GMT_MSG_VERBOSE, "Plotting SAC file %d: %s\n", n, L[n].file);
-		
+ 		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "Plotting SAC file %d: %s\n", n, L[n].file);
+
 		/* -T: determine the reference time for all times in pssac */
 		tref = 0.0;
 		if (Ctrl->T.active) {
@@ -675,10 +701,10 @@ int GMT_pssac (void *V_API, int mode, void *args) {	/* High-level function that
 		y = gmt_M_memory (GMT, NULL, hd.npts, double);
 
 		if (gmt_M_is_linear(GMT)) dt = hd.delta;
-		else if (Ctrl->S.active) dt = hd.delta/Ctrl->S.sec_per_measure;
+		else if (Ctrl->S.active) dt = hd.delta * Ctrl->S.factor;
 		else {
 			GMT_Report (API, GMT_MSG_NORMAL, "Error: -S option is needed in geographic plots.\n");
-			gmt_M_free(GMT, x);		gmt_M_free(GMT, y);
+			gmt_M_free(GMT, x);		gmt_M_free(GMT, y);		gmt_M_free (GMT, L);
 			gmt_M_free(GMT, data);
 			Return(EXIT_FAILURE);
 		}
@@ -732,10 +758,15 @@ int GMT_pssac (void *V_API, int mode, void *args) {	/* High-level function that
 		}
 
 		/* Default to plot trace at station locations on geographic maps */
-		if (!gmt_M_is_linear(GMT) && L[n].position == false) {
-			L[n].position = true;
-			gmt_geo_to_xy (GMT, hd.stlo, hd.stla, &L[n].x, &L[n].y);
+		if (!gmt_M_is_linear(GMT)) {
+			/* Default to plot trace at station locations on geographic maps */
+			if (L[n].position == false) {
+				L[n].position = true;
+				L[n].x = hd.stlo;
+				L[n].y = hd.stla;
+			}
 			GMT_Report (API, GMT_MSG_VERBOSE, "=> %s: Geographic location: (%g, %g)\n", L[n].file, hd.stlo, hd.stla);
+			gmt_geo_to_xy (GMT, L[n].x, L[n].y, &L[n].x, &L[n].y);
 		}
 
 		if (L[n].position) {   /* position (X0,Y0) on plots */
@@ -814,33 +845,6 @@ int GMT_pssac (void *V_API, int mode, void *args) {	/* High-level function that
 		GMT_Report (API, GMT_MSG_LONG_VERBOSE, "=> %s: after scaling and shifting : xmin=%g xmax=%g ymin=%g ymax=%g\n",
 		                                       L[n].file, x[0], x[hd.npts-1], hd.depmin, hd.depmax);
 
-		if (gmt_M_is_linear(GMT)) {
-			GMT->current.plot.n = gmt_geo_to_xy_line (GMT, x, y, hd.npts);
-			xp = GMT->current.plot.x;
-			yp = GMT->current.plot.y;
-			npts = (int)GMT->current.plot.n;
-			plot_pen = GMT->current.plot.pen;
-		}
-		else {
-			xp = x;
-			yp = y;
-			npts = hd.npts;
-			plot_pen = gmt_M_memory (GMT, NULL, npts, unsigned int);
-			plot_pen[0] = PSL_MOVE;
-			free_plot_pen = true;
-		}
-
-		/* plot trace */
-		if (L[n].custom_pen) {
-			current_pen = L[n].pen;
-			gmt_setpen (GMT, &L[n].pen);
-		}
-		gmt_plot_line (GMT, xp, yp, plot_pen, npts, current_pen.mode);
-		if (L[n].custom_pen) {
-			current_pen = Ctrl->W.pen;
-			gmt_setpen (GMT, &current_pen);
-		}
-
 		/* paint trace */
 		for (i = 0; i <= 1; i++) { /* 0=positive; 1=negative */
 			if (Ctrl->G.active[i]) {
@@ -873,6 +877,37 @@ int GMT_pssac (void *V_API, int mode, void *args) {	/* High-level function that
 				paint_phase(GMT, Ctrl, PSL, x, y, hd.npts, zero, Ctrl->G.t0[i], Ctrl->G.t1[i], i);
 			}
 		}
+
+		/* don't draw line if -G is used while -W isn't */
+		draw_line = Ctrl->W.active || !(Ctrl->G.active[0] || Ctrl->G.active[1]);
+		/* draw line */
+		if (draw_line) {
+			if (gmt_M_is_linear(GMT)) {
+				GMT->current.plot.n = gmt_geo_to_xy_line (GMT, x, y, hd.npts);
+				xp = GMT->current.plot.x;
+				yp = GMT->current.plot.y;
+				npts = (int)GMT->current.plot.n;
+				plot_pen = GMT->current.plot.pen;
+			}
+			else {
+				xp = x;
+				yp = y;
+				npts = hd.npts;
+				plot_pen = gmt_M_memory (GMT, NULL, npts, unsigned int);
+				plot_pen[0] = PSL_MOVE;
+				free_plot_pen = true;
+			}
+			if (L[n].custom_pen) {
+				current_pen = L[n].pen;
+				gmt_setpen (GMT, &L[n].pen);
+			}
+			gmt_plot_line (GMT, xp, yp, plot_pen, npts, current_pen.mode);
+			if (L[n].custom_pen) {
+				current_pen = Ctrl->W.pen;
+				gmt_setpen (GMT, &current_pen);
+			}
+		}
+
 		gmt_M_free(GMT, x);
 		gmt_M_free(GMT, y);
 		if (free_plot_pen) gmt_M_free(GMT, plot_pen);
